/*  VideoViewer.java
 *
 *	Responsible for creating a panel for operating display of videos.	
 *
 *  Copyright (C) 2002 by Frank McCown
 *  University of Arkansas at Little Rock
 *  July 2002
 */

import javax.swing.*;
import javax.media.*;
import com.sun.media.ui.*;
import javax.media.protocol.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.sql.*;
import java.util.*;

import rowc.*;


public class VideoViewer extends JPanel
{
	public static JCheckBox watchCheckBox;
	public static JLabel lblInfo;
	private static JComboBox locationsComboBox;  // List of available web cams
	
	public static VideoControlPanel videoControl;	
	private static ControllerVideoPanel videoPanel = null;
	private VideoArchivePanel archivePanel;
	
	private static String currentLocation = "";
	private String archiveCurrentLocation = "";
	
	// Indicates if video is currently being played in viewer
	public static boolean videoPlaying = false;  
	
	// Indicates if video is currently being recorded
	public static boolean recording = false;

	public static rowc.CamManager camManager;
	
 	public static final int MIN_RECORDING_TIME = 1;  // Min recording time in seconds
 	public static final int MAX_RECORDING_TIME = 30;
 	
 	
 	
	public VideoViewer()
	{		
		// Register callback so addWebCam and removeWebCam methods are called by Server
		CamControllerCallback callback = (new CamControllerCallbackImpl())._this(CamController.orb);
		CamController.regObject.addCamController(CamController.hostName, callback);
		
		MediaArchiveCallback mediaCallback = (new MediaArchiveCallbackImpl())._this(CamController.orb);
		CamController.mediaStore.addMediaArchiveListener(CamController.hostName, mediaCallback);
		
		setLayout(new GridBagLayout());

		//lblItem = new JLabel();
		videoPanel = new ControllerVideoPanel();
		JScrollPane scroller = new JScrollPane(videoPanel);


		lblInfo = new JLabel();
		lblInfo.setForeground(Color.red);
				
		locationsComboBox = new JComboBox();
		populateLocationsComboBox();
				
		locationsComboBox.addItemListener(
			new ItemListener()
			{
				public void itemStateChanged(ItemEvent e)
				{
					if (e.getStateChange() == ItemEvent.SELECTED)
					{
						//stopCamViewer();
						if (videoPlaying)
							watchCheckBox.setSelected(false);
						
						Object obj = locationsComboBox.getSelectedItem();
						if (obj != null)
							currentLocation = obj.toString();
						else
						  currentLocation = "";
						
						System.out.println("Selected location " + currentLocation);
						
						lblInfo.setText("");
					}
				}
			}
		);

		// Movement checkbox
		watchCheckBox = new JCheckBox("Watch", false);	
		watchCheckBox.addItemListener(new ItemListener()
		{
			public void itemStateChanged(ItemEvent e)
			{
				//System.out.println("Watch itemStateChanged called.");
				if (e.getStateChange() == ItemEvent.SELECTED)
				{
					// Force to repaint?
			
					Object obj = locationsComboBox.getSelectedItem();
					if (obj != null)
					{
						currentLocation = obj.toString();
						if (!startCamViewer())
							watchCheckBox.setSelected(false);
					}
					else
					{
						// This could happen if location box is empty
					  currentLocation = "";
						showError("There is no location selected.");
						watchCheckBox.setSelected(false);
					}
				}
				else if (e.getStateChange() == ItemEvent.DESELECTED)
					stopCamViewer();
			}
		});	
			
		archivePanel = new VideoArchivePanel();
		videoControl = new VideoControlPanel();
	
		// GridBagConstraints(int gridx, int gridy, int gridwidth, int gridheight, double weightx, double weighty, 
		//   int anchor, int fill, Insets insets, int ipadx, int ipady) 
		
		// Add components to container
		add(scroller, new GridBagConstraints(0, 0, 1, 4, 1.0, 0.9,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 217, 202));
		add(new JLabel("Location:"), new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		add(locationsComboBox, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
		add(watchCheckBox, new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		add(videoControl, new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		add(lblInfo, new GridBagConstraints(0, 6, 2, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		add(archivePanel, new GridBagConstraints(0, 4, 2, 1, 1.0, 1.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 0, 0, 5), 0, 0));
	}

	public void startRecording(int maxTime)
	{
		// Start recording and have it sent here
		
		if (!getCamManagerForLocation())
			return;
		
		recording = true;		
		lblInfo.setText("Recording...");
		
		// Register callback so newRecording method is called
		CamVideoCallback callback = (new CamVideoCallbackImpl())._this(CamController.orb);
				
		if (!camManager.startRec(maxTime, callback))
		{
			showError("Unable to start recording... Cam Processor is busy.");
			lblInfo.setText("");
			return;
		}
	}

	private void stopRecording()
	{
		// Tell cam manager to stop recording and store video on Server.
		camManager.stopRec();
	}
	
	public static void terminate()
	{
		// Notify Server that we're terminating so it takes us off its notification list
		CamController.regObject.removeCamController(CamController.hostName);
	}
	
	private boolean startCamViewer()
	{
		videoPlaying = false;
		
		if (!getCamManagerForLocation())
			return false;
				
		lblInfo.setText("Loading media...");
		
		try
		{
			// Register callback so videoAvailable method is called
			CamVideoCallback callback = (new CamVideoCallbackImpl())._this(CamController.orb);
					
			// Tell cam to start RTP transmission to this computer.
			if (!camManager.startTransmission(CamController.hostName, callback))
			{
				showError("Could not watch because Cam Controller is busy.");
				watchCheckBox.setSelected(false);
				lblInfo.setText("");
				return false;
			}
		}
		catch (org.omg.CORBA.TRANSIENT e)
		{
			e.printStackTrace();
			showError("The web cam is unavailable.  Unable to view video.");
			lblInfo.setText("");
			return false;
		}
		
		return true;
	}
	
	public static void stopCamViewer()
	{
		System.out.println("stopCamViewer");
		
		try
		{
			if (camManager != null && videoPlaying)
			{
				System.out.println("STOPPING TRANSMISSION");
				videoPanel.stop();
				camManager.stopTransmission();
				
				videoPlaying = false;
			}
		}
		catch (org.omg.CORBA.TRANSIENT ex)
		{
			System.out.println(ex);
			showError("It appears that the CamManager being used is no longer available.");
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	
	}
	
	public static boolean getCamManagerForLocation()
	{
		try 
		{
			// Find CamManager for specific location; Obj ref is stored in Naming Service
	    org.omg.CosNaming.NameComponent[] camManagerNC = new org.omg.CosNaming.NameComponent[1];
	    String camName = "CamManager_" + currentLocation;
	    camManagerNC[0] = new org.omg.CosNaming.NameComponent(camName,"");
	    camManager = rowc.CamManagerHelper.narrow(CamController.nc.resolve(camManagerNC));
	 	}
	 	catch (org.omg.CosNaming.NamingContextPackage.NotFound ex)
	 	{
	 		showError("Error retrieving media from the selected location.\n" +
					"Make sure the cam software is running for the selected web cam.");
			return false;
	 	}
	 	catch (Exception ex)
	 	{
	 		ex.printStackTrace();
			showError("Error retrieving media from the selected location.");
			return false;
	 	}
	 	return true;
	}	
	 	

	private void populateLocationsComboBox()
	{
			// Location combo box- Retrieve locations from DB
		if (CamController.connectedToServer)
		{
			try	
			{
				System.out.println("Retrieving locations from server...");
				String[] loc = CamController.regObject.getLocations();
							
				// If first slot is empty, there are no locations available
				if (!loc[0].equals(""))
				{
					locationsComboBox = new JComboBox(loc);
					currentLocation = loc[0];
				}
			}
			catch (Exception e)	
			{
				System.out.println("Unable to get locations from Server.");
				lblInfo.setText("No locations available.");
			}
		}
	}
		
	private static boolean locationIsKnown(String location)
	{
		// Return true if location is already known about (in combo box)
		for (int i = 0; i < locationsComboBox.getItemCount(); i++)
		{
			String loc = (String) locationsComboBox.getItemAt(i);
			if (location.compareToIgnoreCase(loc) == 0)
				return true;
		}
		return false;		
	}

	
	public static void showError(String error)
	{
		JOptionPane.showMessageDialog(null, error, "Cam Controller Error",
						JOptionPane.ERROR_MESSAGE);
	}
		


	/***************************************************************************
	 *
	 * Inner class responsible for GUI for controlling video portion of web cam
	 *
	 ***************************************************************************/
	
	class VideoControlPanel extends JPanel
	{
		private JButton openFileButton, startButton, stopButton;
		private JTextField maxRecTimeTextField;
		
		private FileDialog fd = null;
		
		
		public VideoControlPanel()
		{
			setLayout(new GridBagLayout());
								
			ControlButtonHandler handler = new ControlButtonHandler();
			
			startButton = new JButton("Start");
			startButton.addActionListener(handler);
			
			stopButton = new JButton("Stop");
			stopButton.addActionListener(handler);
			
			// Allow user to enter maximum number of seconds to record
			maxRecTimeTextField = new JTextField("4", 4);
		
			openFileButton = new JButton("Open File...");
			openFileButton.addActionListener(handler);
	
			
			add(new JLabel("Recording:"), new GridBagConstraints(0, 0, 2, 1, 0.0, 0.0,
	            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		
			add(startButton, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
	            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 10, 0), 0, 0));
			add(stopButton, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
	            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 10, 0), 0, 0));
		
			add(new JLabel("Max recording time:"), new GridBagConstraints(0, 2, 2, 1, 0.0, 0.0,
	            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		
			add(maxRecTimeTextField, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
	            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 10, 0), 0, 0));
			add(new JLabel("Seconds"), new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0,
	            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 3, 10, 0), 0, 0));
		
			add(openFileButton, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0,
	            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
	
		}
				
		
		public int getMaxRecTime()
		{
			int time;
			
			try
			{
				time = Integer.parseInt(maxRecTimeTextField.getText());
			}
			catch (NumberFormatException e)
			{
				time = MIN_RECORDING_TIME - 1;
			}
		
			return time;
		}
		
		private void openFile() 
		{
			if (fd == null) 
			{
				// "new Frame" should really be parent's frame, but I'm not sure how
				// to do it, and this seems to work anyway.
				
				fd = new FileDialog(new Frame(), "Open File",
					       FileDialog.LOAD);
				//fd.setDirectory("/movies");
		  }
		  fd.show();
		  if (fd.getFile() != null) 
		  {
				String filename = fd.getDirectory() + fd.getFile();
				videoPanel.play("file:" + filename);
			}
	  }
	  
	  // Called when button is pressed
		private class ControlButtonHandler implements ActionListener
		{
			public void actionPerformed(ActionEvent ae)
			{
				Object btn = ae.getSource();
					
				if (btn == startButton)
				{
					// Start recording
					int maxTime = videoControl.getMaxRecTime();
					if (maxTime < MIN_RECORDING_TIME || maxTime > MAX_RECORDING_TIME)
					{
						showError("Please enter a time value between " + MIN_RECORDING_TIME + " and " +
							MAX_RECORDING_TIME + ".");
						maxRecTimeTextField.requestFocus();
						maxRecTimeTextField.selectAll();
					}
					else
						startRecording(maxTime);				
				}
				else if (btn == stopButton)
				{
					if (recording)
						stopRecording();
				}
				else if (btn == openFileButton)
				  openFile();
			}
		}
	}	// end VideoControlPanel class

	/********************************************************************

		Inner class for displaying all archived media.
		
	********************************************************************/
	
	class VideoArchivePanel extends JPanel
	{
		private JButton viewButton, deleteButton;
		private JComboBox archiveLocationsComboBox;
		private VideoTable videoTable = new VideoTable();
		
		
		public VideoArchivePanel()
		{
			setLayout(new GridBagLayout());
					
			ArchiveButtonHandler handler = new ArchiveButtonHandler();
		
			viewButton = new JButton("View");
			viewButton.addActionListener(handler);
			deleteButton = new JButton("Delete");
			deleteButton.addActionListener(handler);
			
			archiveLocationsComboBox = new JComboBox();
			archiveLocationsComboBox.addItemListener(
				new ItemListener()
				{
					public void itemStateChanged(ItemEvent e)
					{
						Object obj = archiveLocationsComboBox.getSelectedItem();
						if (obj != null)
						{
							archiveCurrentLocation = obj.toString();
							System.out.println("call populateVideoTable with archiveCurrentLocation= "+archiveCurrentLocation);
							
							// Get list of videos from Server for the selected location
							populateVideoTable();  
						}
						else
						  archiveCurrentLocation = "";
						
						System.out.println("Selected archive location " + archiveCurrentLocation);
					}
				}
			);
			
			populateArchiveLocationsComboBox();
			
			JScrollPane scrollPane = new JScrollPane(videoTable);
			
			// GridBagConstraints(int gridx, int gridy, int gridwidth, int gridheight, double weightx, double weighty, 
			//   int anchor, int fill, Insets insets, int ipadx, int ipady) 

			add(new JLabel("Video Archive:"), new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
	            GridBagConstraints.NORTHWEST, GridBagConstraints.NORTH, new Insets(0, 10, 10, 0), 0, 0));
		
			add(archiveLocationsComboBox, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
	            GridBagConstraints.WEST, GridBagConstraints.NORTH, new Insets(0, 10, 10, 0), 0, 0));
			add(viewButton, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
	            GridBagConstraints.WEST, GridBagConstraints.NORTH, new Insets(0, 10, 10, 0), 0, 0));
			add(deleteButton, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0,
	            GridBagConstraints.WEST, GridBagConstraints.NORTH, new Insets(0, 10, 10, 0), 0, 0));
		
			add(scrollPane, new GridBagConstraints(0, 1, 5, 1, 2.0, 2.0,
	            GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(0, 10, 10, 0), 0, 0));
	
		}
			// Called when button is pressed
		private class ArchiveButtonHandler implements ActionListener
		{
			public void actionPerformed(ActionEvent ae)
			{
				Object btn = ae.getSource();
					
				if (btn == viewButton)
				{
					System.out.println("viewButton");			
					
					// Get video clip from Server using RTP
					int selectedRow = videoTable.getSelectedRow();
					if (selectedRow == -1)
					{
						showError("Please select a row from the video archive table to view.");
						return;
					}
					
					// Can't be watching anything now
					if (watchCheckBox.isSelected())
						watchCheckBox.setSelected(false);
					
					String videoName = videoTable.getVideoName(selectedRow);
					String rtp = CamController.mediaStore.getVideoStream(archiveCurrentLocation, 
						videoName, CamController.hostName);
					
					videoPlaying = true;

					// Use RTPSessionManager so it will know when video is no longer available
					videoPanel.playRTP(rtp);
					//videoPanel.play(rtp);
				}
				else if (btn == deleteButton)
				{
					//System.out.println("deleteButton");
					
					// Delete selected video from database and from video dir
					
					int selectedRows[] = videoTable.getSelectedRows();
					if (selectedRows.length == 0)
					{
						showError("Please select a row from the video archive table to delete.");
						return;
					}					
					
					if (JOptionPane.showConfirmDialog(null, 
						"Are you sure you want to permanently delete the selected video(s)?", 
						"Cam Controller",
						JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
					{
						// Delete all selected videos
						
						String videoNames[] = new String[selectedRows.length];
						for (int i=0; i < selectedRows.length; i++)
						{												
							videoNames[i] = videoTable.getVideoName(selectedRows[i]);
							System.out.println("About to delete: " + videoNames[i]);
						}
						
						boolean success = CamController.mediaStore.deleteVideos(archiveCurrentLocation, videoNames);
						if (!success)
							showError("Error encountered while trying to delete the selected video.");		
					}
				}
			}
		}
		
		public void updateLocationList(String location)
		{
			System.out.println("updateLocationList called");
			
			// Add this location to combo box if it's not already there
			int items = archiveLocationsComboBox.getItemCount();
			boolean found = false;
			for (int i = 0; i < items && !found; i++)
			{
				String loc = (String) archiveLocationsComboBox.getItemAt(i);
				if (loc.equals(location))
					found = true;
			}
			
			if (!found)
			{
				System.out.println("Adding new location: " + location);
				archiveLocationsComboBox.addItem(location);
			}
			else if (archiveCurrentLocation.equals(location))
				populateVideoTable();  // Add new video to the list.  
		}
		
		private void populateArchiveLocationsComboBox()
		{
			// Location combo box- Retrieve locations from DB
			if (CamController.connectedToServer)
			{
				String[] loc = null;
				
				try	
				{
					System.out.println("Retrieving archive locations from server...");
					loc = CamController.mediaStore.getArchiveLocations();
								
					if (loc.length != 0)
					{
						if (!loc[0].equals(""))
						{
							for (int i=0; i < loc.length; i++)
								archiveLocationsComboBox.addItem(loc[i]);

							archiveCurrentLocation = (String) archiveLocationsComboBox.getItemAt(0);
							
							// Show videos for this location
							populateVideoTable();
						}
					}
					else
					{
						lblInfo.setText("No archived video available yet.");
					}
				}
				catch (rowc.MediaStorePackage.LocationsUnavailable e)	
				{
					System.out.println("Unable to get locations from database.");
					showError("Error retrieving locations from the Server.");
				}
				catch (Exception e)	
				{
					System.out.println("Unable to get locations from Server.");
					e.printStackTrace();
					showError("Error retrieving locations from the Server.");
				}
			}
		}
		
		public void populateVideoTable()
		{
			System.out.println("populateVideoTable called");
			if (CamController.connectedToServer)
			{
				videoTable.populateVideoTable(archiveCurrentLocation, CamController.mediaStore);
				
				// update UI
				//videoTable.repaint();
			}
		}
	}
	
	/*************************************************************************************
	 *
	 *  Inner class for notification of archive media events.
	 *
	 ************************************************************************************/
	
	class MediaArchiveCallbackImpl extends MediaArchiveCallbackPOA
	{
		public synchronized void addVideo(String location)
		{
			System.out.println("\nvideoAdded called for loc=" + location + "\n");
			archivePanel.updateLocationList(location);
		}
		
    public synchronized void removeVideo(String location)
  	{
  		System.out.println("\nvideoDeleted called for loc=" + location + "\n");
  		archivePanel.updateLocationList(location);
  	}
	}
	
	
	/**************************************************************************************
	 *
	 *  Inner class for callbacks when receiving notification of new/removed web cam locations.
	 *
	 *
	 *************************************************************************************/
	
	public class CamControllerCallbackImpl extends rowc.CamControllerCallbackPOA 
	{
		// Server notifying us that new web cam is available.  Add it to our list of locations.
		public synchronized void addWebCam(String location)
		{
			System.out.println("addWebCam is adding " + location + " to locations vector.");
						
			locationsComboBox.addItem(location);  // Automatically adds to locations vector
			locationsComboBox.repaint();
		}
		
		// Server notifying us that web cam is no longer available.  Remove from location list.
	  public synchronized void removeWebCam(String location)
	  {
	  	System.out.println("removeWebCam removing location: " + location);
	  	
	  	locationsComboBox.removeItem(location);  // Automatically removes from locations vector
	  	locationsComboBox.setSelectedItem("");  // Deselects item
	  	locationsComboBox.repaint();
	  }
		
	} // end CamControllerCallbackImpl class
	
	/*************************************************************************************
	
	Inner class for notification of new videos.
	
	*************************************************************************************/
	
	class CamVideoCallbackImpl extends CamVideoCallbackPOA
	{
		// Called by CamProcessor when video has finished recording to file and has been
		// transmitted to the Server.  This notifies the CamController of the new video
		// so it can tell the server to store it in the database.  The new video is also
		// displayed in the list of archived videos.
		
		public void newRecording(VideoClip video)
		{
			// Right now if location is blank, there's a problem.  Could make more robust by
			// throwing an exception.  ORBacus doesn't like to send "null", so exceptions are the better way to go.
			if (video.location.equals(""))
			{
				System.out.println("Error getting video from exporter.");
				lblInfo.setText("Error getting video");
				recording = false;
				showError("Error retrieving video.");
				return;
			}
			
			System.out.println("Received new video " + video.name + " from location " + video.location);
			
	//		System.out.println("isSelected="+watchCheckBox.isSelected());
	//		System.out.println("videoPlaying="+videoPlaying);
		
			lblInfo.setText("");
			recording = false;
		
			// The CamProcessor sends the video to the MediaServer via sockets but 
			// does not tell the server to store the video since the MediaStore object
			// is not accessible to the CamProcessor.  It is only instantiated by the
			// CamController, and that is why it is responsible for telling the server
			// to store the video.
		
			// Store new video on the Server
		  CamController.mediaStore.storeVideo(video);
		  
		  // Tell VideoArchivePanel to repopulate if this is a new archive location
		  //archivePanel.updateLocationList(video.location);
		}
		
		public void videoAvailable(String url)
		{
			System.out.println("Video available at url: " + url);
				
			System.out.println("isSelected="+watchCheckBox.isSelected());
			System.out.println("videoPlaying="+videoPlaying);
			if (watchCheckBox.isSelected() && videoPlaying == false)
			{
				videoPlaying = true;
				videoPanel.play(url);
			}
				
			lblInfo.setText("");
		}	
	
	}
}	// end VideoViewer class